@@ -53,12 +53,12 @@ class Command(BaseCommand):
         for exp in EXCLUDE:
            filter_re.append(re.compile(exp))
         for k,v in VERSIONS.iteritems():
-            exp = (r'_%s.(%s)') % (k, '|'.join(EXTENSION_LIST))
+            exp = (r'_%s(%s)') % (k, '|'.join(EXTENSION_LIST))
             filter_re.append(re.compile(exp))
         
         # walkt throu the filebrowser directory
         # for all/new files (except file versions itself and excludes)
-        for dirpath,dirnames,filenames in os.walk(path):
+        for dirpath,dirnames,filenames in os.walk(path, followlinks=True):
             for filename in filenames:
                 filtered = False
                 # no "hidden" files (stating with ".")
