@@ -13,7 +13,6 @@ from django.views.decorators.cache import never_cache
 from django.utils.translation import ugettext as _
 from django import forms
 from django.core.urlresolvers import reverse, get_urlconf, get_resolver
-from django.dispatch import Signal
 from django.core.paginator import Paginator, InvalidPage, EmptyPage
 from django.utils.encoding import smart_unicode
 from django.contrib import messages
@@ -29,6 +28,7 @@ from filebrowser.templatetags.fb_tags import query_helper
 from filebrowser.base import FileListing, FileObject
 from filebrowser.decorators import path_exists, file_exists
 from filebrowser.storage import FileSystemStorageMixin, StorageMixin
+from filebrowser import signals
 
 # Add some required methods to FileSystemStorage
 if FileSystemStorageMixin not in FileSystemStorage.__bases__:
@@ -267,10 +267,6 @@ class FileBrowserSite(object):
             'breadcrumbs_title': "",
             'filebrowser_site': self
         }, context_instance=Context(request, current_app=self.name))
-    
-    # mkdir signals
-    filebrowser_pre_createdir = Signal(providing_args=["path", "name"])
-    filebrowser_post_createdir = Signal(providing_args=["path", "name"])
 
     def createdir(self, request):
         """
@@ -285,11 +281,11 @@ class FileBrowserSite(object):
             if form.is_valid():
                 server_path = os.path.join(path, form.cleaned_data['name'])
                 try:
-                    self.filebrowser_pre_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'])
+                    signals.filebrowser_pre_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'], site=self)
                     self.storage.makedirs(server_path)
                     # os.mkdir(server_path)
                     # os.chmod(server_path, 0775) # ??? PERMISSIONS
-                    self.filebrowser_post_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'])
+                    signals.filebrowser_post_createdir.send(sender=request, path=server_path, name=form.cleaned_data['name'], site=self)
                     messages.add_message(request, messages.SUCCESS, _('The Folder %s was successfully created.') % form.cleaned_data['name'])
                     redirect_url = reverse("filebrowser:fb_browse", current_app=self.name) + query_helper(query, "ot=desc,o=date", "ot,o,filter_type,filter_date,q,p")
                     return HttpResponseRedirect(redirect_url)
@@ -362,10 +358,6 @@ class FileBrowserSite(object):
             'filebrowser_site': self
         }, context_instance=Context(request, current_app=self.name))
 
-    # delete signals
-    filebrowser_pre_delete = Signal(providing_args=["path", "name"])
-    filebrowser_post_delete = Signal(providing_args=["path", "name"])
-
     def delete(self, request):
         """
         Delete existing File/Directory.
@@ -376,10 +368,10 @@ class FileBrowserSite(object):
         
         if request.GET:
             try:
-                self.filebrowser_pre_delete.send(sender=request, path=fileobject.path, name=fileobject.filename)
+                signals.filebrowser_pre_delete.send(sender=request, path=fileobject.path, name=fileobject.filename, site=self)
                 fileobject.delete_versions()
                 fileobject.delete()
-                self.filebrowser_post_delete.send(sender=request, path=fileobject.path, name=fileobject.filename)
+                signals.filebrowser_post_delete.send(sender=request, path=fileobject.path, name=fileobject.filename, site=self)
                 messages.add_message(request, messages.SUCCESS, _('Successfully deleted %s') % fileobject.filename)
             except OSError, (errno, strerror):
                 # TODO: define error-message
@@ -387,13 +379,6 @@ class FileBrowserSite(object):
         redirect_url = reverse("filebrowser:fb_browse", current_app=self.name) + query_helper(query, "", "filename,filetype")
         return HttpResponseRedirect(redirect_url)
 
-    # rename signals
-    filebrowser_pre_rename = Signal(providing_args=["path", "name", "new_name"])
-    filebrowser_post_rename = Signal(providing_args=["path", "name", "new_name"])
-
-    filebrowser_actions_pre_apply = Signal(providing_args=['action_name', 'fileobjects',])
-    filebrowser_actions_post_apply = Signal(providing_args=['action_name', 'filebjects', 'result'])
-
     def detail(self, request):
         """
         Show detail page for a file.
@@ -415,16 +400,16 @@ class FileBrowserSite(object):
                     if action_name:
                         action = self.get_action(action_name)
                         # Pre-action signal
-                        self.filebrowser_actions_pre_apply.send(sender=request, action_name=action_name, fileobject=[fileobject])
+                        signals.filebrowser_actions_pre_apply.send(sender=request, action_name=action_name, fileobject=[fileobject], site=self)
                         # Call the action to action
                         action_response = action(request=request, fileobjects=[fileobject])
                         # Post-action signal
-                        self.filebrowser_actions_post_apply.send(sender=request, action_name=action_name, fileobject=[fileobject], result=action_response)
+                        signals.filebrowser_actions_post_apply.send(sender=request, action_name=action_name, fileobject=[fileobject], result=action_response, site=self)
                     if new_name != fileobject.filename:
-                        self.filebrowser_pre_rename.send(sender=request, path=fileobject.path, name=fileobject.filename, new_name=new_name)
+                        signals.filebrowser_pre_rename.send(sender=request, path=fileobject.path, name=fileobject.filename, new_name=new_name, site=self)
                         fileobject.delete_versions()
                         self.storage.move(fileobject.path, os.path.join(fileobject.head, new_name))
-                        self.filebrowser_post_rename.send(sender=request, path=fileobject.path, name=fileobject.filename, new_name=new_name)
+                        signals.filebrowser_post_rename.send(sender=request, path=fileobject.path, name=fileobject.filename, new_name=new_name, site=self)
                         messages.add_message(request, messages.SUCCESS, _('Renaming was successful.'))
                     if isinstance(action_response, HttpResponse):
                         return action_response
@@ -464,31 +449,24 @@ class FileBrowserSite(object):
             'filebrowser_site': self
         }, context_instance=Context(request, current_app=self.name))
 
-    # upload signals
-    filebrowser_pre_upload = Signal(providing_args=["path", "file"])
-    filebrowser_post_upload = Signal(providing_args=["path", "file"])
-
     def _upload_file(self, request):
         """
         Upload file to the server.
         """
         if request.method == "POST":
+            folder = request.GET.get('folder', '')
+
             if request.is_ajax(): # Advanced (AJAX) submission
-                folder = request.GET.get('folder')
                 filedata = ContentFile(request.raw_post_data)
-                try:
-                    filedata.name = convert_filename(request.GET['qqfile'])
-                except KeyError:
-                    return HttpResponseBadRequest('Invalid request! No filename given.')
             else: # Basic (iframe) submission
-                # TODO: This needs some attention, do we use this at all?
-                folder = request.POST.get('folder')
-                if len(request.FILES) == 1:
-                    filedata = request.FILES.values()[0]
-                else:
+                if len(request.FILES) != 1:
                     raise Http404('Invalid request! Multiple files included.')
-                # filedata.name = convert_filename(upload.name)
-                filedata.name = convert_filename(request.POST.get('file_name'))
+                filedata = request.FILES.values()[0]
+
+            try:
+                filedata.name = convert_filename(request.GET['qqfile'])
+            except KeyError:
+                return HttpResponseBadRequest('Invalid request! No filename given.')
 
             fb_uploadurl_re = re.compile(r'^.*(%s)' % reverse("filebrowser:fb_upload", current_app=self.name))
             folder = fb_uploadurl_re.sub('', folder)
@@ -502,15 +480,17 @@ class FileBrowserSite(object):
                 ret_json = {'success': False, 'filename': filedata.name}
                 return HttpResponse(json.dumps(ret_json)) 
             
-            self.filebrowser_pre_upload.send(sender=request, path=request.POST.get('folder'), file=filedata)
+            signals.filebrowser_pre_upload.send(sender=request, path=request.POST.get('folder'), file=filedata, site=self)
             uploadedfile = handle_file_upload(path, filedata, site=self)
             
-            if file_already_exists:
+            if file_already_exists and OVERWRITE_EXISTING:
                 old_file = smart_unicode(file_name)
                 new_file = smart_unicode(uploadedfile)
                 self.storage.move(new_file, old_file, allow_overwrite=True)
+            else:
+                file_name = smart_unicode(uploadedfile)
             
-            self.filebrowser_post_upload.send(sender=request, path=request.POST.get('folder'), file=FileObject(smart_unicode(file_name), site=self))
+            signals.filebrowser_post_upload.send(sender=request, path=request.POST.get('folder'), file=FileObject(smart_unicode(file_name), site=self), site=self)
             
             # let Ajax Upload know whether we saved it or not
             ret_json = {'success': True, 'filename': filedata.name}
